/***************************************************************************

 Author        : Mariah Nie - CAST

 Date          : Jun 12th, 2009

 File          : ADF4350_IO.c

 Hardware      : ADuC7026 and ADF4350

 Description   : Use the GPIO to simulate the SPI communication of ADF4350
	
***************************************************************************/
#include <ADuC7026.h>
#include "ADF4350_IO.h"

#define SET_LE()		GP4DAT = (GP4DAT | 0x00080000)	//P4.3->LE
#define CLR_LE()		GP4DAT = (GP4DAT & 0xFFF7FFFF)

#define	SET_SCL()		GP4DAT = (GP4DAT | 0x00100000)	//P4.4->SCL
#define	CLR_SCL()		GP4DAT = (GP4DAT & 0xffefffff)

#define SET_DATA()		GP4DAT = (GP4DAT | 0x00200000)	//P4.5->DATA
#define CLR_DATA()		GP4DAT = (GP4DAT & 0xffdfffff)

#define SET_DATA_OUT()	GP4DAT = GP4DAT | 0x20000000;
#define SET_DATA_IN()	GP4DAT = GP4DAT & 0xDFFFFFFF;

void delay (int length)
{
	while (length >0)
    	length--;
}

//---------------------------------
//void WriteToADF4350(unsigned char count,unsigned char *buf);
//---------------------------------
//Function that writes to the ADF4350 via the SPI port. 
//--------------------------------------------------------------------------------
void WriteToADF4350(unsigned char count, unsigned char *buf)
{
	unsigned	char	ValueToWrite = 0;
    unsigned	char	i = 0;
	unsigned	char	j = 0;
	
	SET_DATA_OUT();
	
	delay(1);
	CLR_SCL();
	CLR_LE();
	delay(1);

	for(i=count;i>0;i--)
 	{
	 	ValueToWrite = *(buf + i - 1);
		for(j=0; j<8; j++)
		{
			if(0x80 == (ValueToWrite & 0x80))
			{
				SET_DATA();	  //Send one to SDO pin
			}
			else
			{
				CLR_DATA();	  //Send zero to SDO pin
			}
			delay(1);
			SET_SCL();
			delay(1);
			ValueToWrite <<= 1;	//Rotate data
			CLR_SCL();
		}
	}
	CLR_DATA();
	delay(1);
	SET_LE();
	delay(1);
	CLR_LE();
}


//---------------------------------
//void ReadFromADF4350(unsigned char count,unsigned char *buf)
//---------------------------------
//Function that reads from the ADF4350 via the SPI port. 
//--------------------------------------------------------------------------------
void ReadFromADF4350(unsigned char count, unsigned char *buf)
{
	unsigned	char	i = 0;
	unsigned	char	j = 0;
	unsigned	int  	iTemp = 0;
	unsigned	char  	RotateData = 0;

	SET_DATA_IN();
	
	delay(1);
	CLR_SCL();
	CLR_LE();
	delay(1);

	for(j=count; j>0; j--)
	{
		for(i=0; i<8; i++)
		{
			RotateData <<= 1;		//Rotate data
			delay(1);
			iTemp = GP4DAT;			//Read DATA of ADF4350
			SET_SCL();	
			if(0x00000020 == (iTemp & 0x00000020))
			{
				RotateData |= 1;	
			}
			delay(1);
			CLR_SCL();
		}
		*(buf + j - 1)= RotateData;
	}	 
	delay(1);
	SET_LE();
	delay(1);
	CLR_LE();
} 

